package djmil.cordacheckers.cordaclient;

import java.io.IOException;
import java.security.KeyManagementException;
import java.security.KeyStoreException;
import java.security.NoSuchAlgorithmException;
import java.security.cert.CertificateException;

import javax.net.ssl.SSLContext;

import org.apache.hc.client5.http.impl.classic.CloseableHttpClient;
import org.apache.hc.client5.http.impl.classic.HttpClients;
import org.apache.hc.client5.http.impl.io.PoolingHttpClientConnectionManager;
import org.apache.hc.client5.http.socket.ConnectionSocketFactory;
import org.apache.hc.client5.http.socket.PlainConnectionSocketFactory;
import org.apache.hc.client5.http.ssl.NoopHostnameVerifier;
import org.apache.hc.client5.http.ssl.SSLConnectionSocketFactory;
import org.apache.hc.core5.http.config.Registry;
import org.apache.hc.core5.http.config.RegistryBuilder;
import org.apache.hc.core5.ssl.SSLContexts;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.core.io.Resource;
import org.springframework.http.client.HttpComponentsClientHttpRequestFactory;
import org.springframework.web.client.RestTemplate;

@Configuration
public class RestTemplateConfiguration {

    @Value("${trust.store}")
    private Resource trustStore;

    @Value("${trust.store.password}")
    private String trustStorePassword;

    @Bean
    RestTemplate restTemplate() throws KeyManagementException, NoSuchAlgorithmException, KeyStoreException, CertificateException, IOException {
      
		System.out.println("Setting UP");

		final SSLContext sslContext = SSLContexts.custom()
			.loadTrustMaterial(trustStore.getURL(), trustStorePassword.toCharArray())
			.build();

		final SSLConnectionSocketFactory sslsf = new SSLConnectionSocketFactory(sslContext, NoopHostnameVerifier.INSTANCE);
		final Registry<ConnectionSocketFactory> socketFactoryRegistry = RegistryBuilder.<ConnectionSocketFactory> create()
			.register("https", sslsf)
			.register("http", new PlainConnectionSocketFactory())
			.build();

		//final BasicHttpClientConnectionManager connectionManager = 
		// 	new BasicHttpClientConnectionManager(socketFactoryRegistry);
		final PoolingHttpClientConnectionManager connectionManager = 
			new PoolingHttpClientConnectionManager(socketFactoryRegistry);
		// connectionManager.setMaxTotal(100);
        // connectionManager.setDefaultMaxPerRoute(6);

		final CloseableHttpClient httpClient = HttpClients.custom()
			.setConnectionManager(connectionManager)
			.build();

		final HttpComponentsClientHttpRequestFactory requestFactory =
			new HttpComponentsClientHttpRequestFactory(httpClient);
		//requestFactory.setReadTimeout(readTimeout);
		//requestFactory.setConnectTimeout(connectTimeout);

		return new RestTemplate(requestFactory);

        // String BASE_URI_TEMPLATE = "http://localhost:8080";

        // DefaultUriBuilderFactory uriBuilderFactory = new DefaultUriBuilderFactory(BASE_URI_TEMPLATE);

        // RestTemplate restTemplate = new RestTemplate();
        // restTemplate.setUriTemplateHandler(uriBuilderFactory);
        // restTemplate.getForObject("/test", String.class); // like you may have used earlier
    }
}
